Package: gena
Type: Package
Title: Genetic Algorithm and Particle Swarm Optimization
Version: 1.0.1
Date: 2026-04-08
Authors@R: c(person("Bogdan", "Potanin", email = "bogdanpotanin@gmail.com", role = c("aut", "cre", "ctb")), person("Sofiia", "Dolgikh", email = "sidolgikh@gmail.com", role = c("ctb")))
Description: Implements genetic algorithm and particle swarm algorithm for real-valued functions. Various modifications (including hybridization and elitism) of these algorithms are provided. Implemented functions are based on ideas described in S. Katoch, S. Chauhan, V. Kumar (2020) <doi:10.1007/s11042-020-10139-6> and M. Clerc (2012) <https://hal.science/hal-00764996>.
Imports: Rcpp (>= 1.0.6)
LinkingTo: Rcpp, RcppArmadillo
License: GPL (>= 2)
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-04-08 11:13:31 UTC; Bogdan
Author: Bogdan Potanin [aut, cre, ctb],
  Sofiia Dolgikh [ctb]
Maintainer: Bogdan Potanin <bogdanpotanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-09 05:10:39 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-09 07:19:35 UTC; unix
Archs: gena.so.dSYM
