# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @export
mating_tournament <- function(pop_ind, candidates_n, fitness, parents_n) {
    .Call(`_gena_mating_tournament`, pop_ind, candidates_n, fitness, parents_n)
}

